/* $Id: espda_line.h 234525 2019-01-10 16:45:58Z gcupani $
 *
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: gcupani $
 * $Date: 2019-01-10 16:45:58 +0000 (Thu, 10 Jan 2019) $
 * $Revision: 234525 $
 * $Name:  $
 */

#ifndef ESPDA_LINE_H
#define ESPDA_LINE_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_line_idmult(A, B, C, D)  \
            if (!strcmp(A, B)) {       \
                C = D;                 \
            }                          \

#define espda_line_basic(A)                                    \
            espda_struct_initcol(A, "PEAK",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "WAVEL", CPL_TYPE_DOUBLE); \

#define espda_line_syst(A)                                      \
            espda_line_voigt(A);                                \
            espda_struct_initcol(A, "SYSTID", CPL_TYPE_STRING); \
            espda_struct_initcol(A, "ORDER",  CPL_TYPE_INT);    \

#define espda_line_voigt(A)                                        \
            espda_line_basic(A);                                   \
            espda_struct_initcol(A, "WAVELERR",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "WAVELMIN",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "WAVELMAX",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "LINEID",    CPL_TYPE_STRING); \
            espda_struct_initcol(A, "MULTID",    CPL_TYPE_STRING); \
            espda_struct_initcol(A, "GRPID",     CPL_TYPE_INT);    \
            espda_struct_initcol(A, "LINEZ",     CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "LINEZERR",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "LINEZMIN",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "LINEZMAX",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "LINEZCON",  CPL_TYPE_STRING); \
            espda_struct_initcol(A, "COLDEN",    CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "COLDENERR", CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "COLDENMIN", CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "COLDENMAX", CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "COLDENCON", CPL_TYPE_STRING); \
            espda_struct_initcol(A, "THERB",     CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "THERBERR",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "THERBMIN",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "THERBMAX",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "THERBCON",  CPL_TYPE_STRING); \
            espda_struct_initcol(A, "TURBB",     CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "TURBBERR",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "TURBBMIN",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "TURBBMAX",  CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "TURBBCON",  CPL_TYPE_STRING); \
            espda_struct_initcol(A, "USE",       CPL_TYPE_STRING); \
            espda_struct_initcol(A, "CONT",      CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "NORM",      CPL_TYPE_DOUBLE); \

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>

#include "espda_dfs.h"
#include "espda_model.h"
#include "espda_param.h"
#include "espda_print.h"
#include "espda_struct.h"

/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_error_code  espda_line_add   (cpl_table   *,
                                  const char  *);

cpl_error_code  espda_line_clean (cpl_table   *,
                                  double);
/*
cpl_table      *espda_line_detect_old(cpl_table   *,
                                  double,
                                  double,
                                  double);
cpl_error_code espda_line_detect_old(cpl_table   *,
                                  double,
                                  double,
                                  double,
                                  cpl_table  **);
*/

cpl_table     *espda_line_detect(cpl_table   *,
                                 cpl_table   *,
                                  double);

cpl_error_code espda_line_fill  (cpl_table   *,
//                                 cpl_table   *,
                                 double,
                                 double,
                                 double      *,
                                 cpl_table  **);

cpl_error_code espda_line_group (cpl_table   *,
                                 cpl_table   *,
                                 //double,
                                 double      *);
/*
cpl_error_code espda_line_group (cpl_table   *);
*/
cpl_error_code espda_line_iden  (cpl_table   *,
                                 cpl_table   *,
                                 double,
                                 cpl_table  **,
                                 cpl_table  **,
                                 cpl_size    *,
                                 cpl_size    *);

cpl_error_code espda_line_ion   (cpl_table   *,
                                 cpl_table   *,
                                 cpl_array  **,
                                 cpl_array  **,
                                 cpl_array  **);

#endif
